unit hhFader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, extctrls;

type
  TTickSide=(tsLeftTop, tsRightBottom, tsBoth, tsNone);
  TOrientation=(foVertical, foHorizontal);
  TFaderChangeEvent = procedure(Sender : TObject; Value : Longint) of object;
  TBevelForm=(bfNone, bfLowered, bfRaised);

  ThhFader = class(TCustomControl)
  private
    { Private-Deklarationen }
    fPosition            : Longint;
    fColor               : TColor;
    fTickColor           : TColor;
    fPaddingTopBottom    : Longint;
    fPaddingLeftRight    : Longint;
    fMin                 : Longint;
    fMax                 : Longint;
    fFrequency           : Longint;
    fTickLength          : Longint;
    fKnobColor           : TColor;
    fKnobBorderColor     : TColor;
    fKnobWidth           : Longint;
    fKnobHeight          : Longint;
    fKnobBorderWidth     : Longint;
    fTickSide            : TTickSide;
    fFaderChangeEvent    : TFaderChangeEvent;
    fShowTickLine        : Boolean;
    fKnobGlyph           : TBitmap;
    fOrientation         : TOrientation;
    fOnMouseDown         : TMouseEvent;
    fOnMouseMove         : TMouseMoveEvent;
    fOnMouseUp           : TMouseEvent;
    fShowArrows          : Boolean;
    fBackGlyph           : TBitmap;
    fScrollStroke        : Integer;
    fBevelInner          : TBevelForm;
    fBevelOuter          : TBevelForm;
    fBevelWidth          : Longint;

    MouseOn              : Boolean;

    procedure SetPosition (Value: Longint);
    procedure SetColor (Value: TColor);
    procedure SetTickColor (Value: TColor);
    procedure SetPaddingTopBottom (Value: Longint);
    procedure SetPaddingLeftRight (Value: Longint);
    procedure SetMin (Value: Longint);
    procedure SetMax (Value: Longint);
    procedure SetFrequency (Value: Longint);
    procedure SetTickLength (Value: Longint);
    procedure SetKnobColor (Value: TColor);
    procedure SetKnobBorderColor (Value: TColor);
    procedure SetKnobWidth (Value: Longint);
    procedure SetKnobHeight (Value: Longint);
    procedure SetKnobBorderWidth (Value: Longint);
    procedure SetTickSide (Value: TTickSide);
    procedure SetShowTickLine (Value: Boolean);
    procedure SetKnobGlyph (Value: TBitmap);
    procedure SetOrientation (Value: TOrientation);
    procedure SetShowArrows (Value: Boolean);
    procedure SetBackGlyph (Value: TBitmap);
    procedure SetScrollStroke (Value: Longint);
    procedure SetBevelInner (Value: TBevelForm);
    procedure SetBevelOuter (Value: TBevelForm);
    procedure SetBevelWidth (Value: Longint);

    procedure UpdatePosition(X, Y: Integer);

  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure FaderChange;
    procedure WMMouseWheel(Var Message: TMessage); message WM_Mousewheel;

  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent); override;

  published
    { Published-Deklarationen }
    property ParentColor;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property Enabled;
    property Position : Longint read fPosition write SetPosition;
    property Color : TColor read fColor write SetColor;
    property TickColor : TColor read fTickColor write SetTickColor;
    property PaddingTopBottom : Longint read fPaddingTopBottom write SetPaddingTopBottom;
    property PaddingLeftRight : Longint read fPaddingLeftRight write SetPaddingLeftRight;
    property Min : Longint read fMin write SetMin;
    property Max : Longint read fMax write SetMax;
    property Frequency : Longint read fFrequency write SetFrequency;
    property TickLength : Longint read fTickLength write SetTickLength;
    property KnobColor : TColor read fKnobColor write SetKnobColor;
    property KnobBorderColor : TColor read fKnobBorderColor write SetKnobBorderColor;
    property KnobWidth : Longint read fKnobWidth write SetKnobWidth;
    property KnobHeight : Longint read fKnobHeight write SetKnobHeight;
    property KnobBorderWidth : Longint read fKnobBorderWidth write SetKnobBorderWidth;
    property TickSide : TTickSide read fTickSide write SetTickSide;
    property OnChange : TFaderChangeEvent read fFaderChangeEvent write fFaderChangeEvent;
    property ShowTickLine : Boolean read fShowTickLine write SetShowTickLine;
    property KnobGlyph: TBitmap read fKnobGlyph write SetKnobGlyph;
    property Orientation: TOrientation read fOrientation write SetOrientation;
    property OnMouseDown: TMouseEvent read fOnMouseDown write fOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read fOnMouseMove write fOnMouseMove;
    property OnMouseUp: TMouseEvent read fOnMouseUp write fOnMouseUp;
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnClick;
    property OnDblClick;
    property ShowArrows: Boolean read fShowArrows write SetShowArrows;
    property BackGlyph: TBitmap read fBackGlyph write SetBackGlyph;
    property ScrollStroke: Longint read fScrollStroke write SetScrollStroke;
    property Anchors;
    property Align;
    property BevelInner: TBevelForm read fBevelInner write SetBevelInner;
    property BevelOuter: TBevelForm read fBevelOuter write SetBevelOuter;
    property BevelWidth: Longint read fBevelWidth write SetBevelWidth;

  end;

procedure Register;

implementation

constructor ThhFader.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
ControlStyle := ControlStyle + [csOpaque];
Width:=40;
Height:=200;
fPosition:=0;
fColor:=clBtnFace;
fTickColor:=clBlack;
fPaddingTopBottom:=6;
fPaddingLeftRight:=4;
fMin:=0;
fMax:=100;
fFrequency:=10;
fTickLength:=10;
fKnobColor:=clBlue;
fKnobBorderColor:=clSilver;
fKnobWidth:=30;
fKnobHeight:=10;
fKnobBorderWidth:=1;
fTickSide:=tsRightBottom;
fShowTickLine:=TRUE;
fKnobGlyph:=TBitmap.Create;
fOrientation:=foVertical;
fShowArrows:=TRUE;
fBackGlyph:=TBitmap.Create;
fScrollStroke:=1;
fBevelInner:=bfNone;
fBevelOuter:=bfRaised;
fBevelWidth:=1;
TabStop:=TRUE;
end;


procedure ThhFader.SetPosition (Value: Longint);
begin
if Value <> fPosition then
 begin
 fPosition:=Value;
 if Value<0 then fPosition:=0;
 if Value<Min then fPosition:=Min;
 if Value>Max then fPosition:=Max;
 Refresh;
 FaderChange;
 end;
end;


procedure ThhFader.SetColor (Value: TColor);
begin
if Value <> fColor then
 begin
 fColor:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetTickColor (Value: TColor);
begin
if Value <> fTickColor then
 begin
 fTickColor:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetPaddingTopBottom (Value: Longint);
begin
if Value <> fPaddingTopBottom then
 begin
 fPaddingTopBottom:=Value;
 if Value<0 then fPaddingTopBottom:=0;
 Refresh;
 end;
end;


procedure ThhFader.SetPaddingLeftRight (Value: Longint);
begin
if Value <> fPaddingLeftRight then
 begin
 fPaddingLeftRight:=Value;
 if Value<0 then fPaddingLeftRight:=0;
 Refresh;
 end;
end;


procedure ThhFader.SetMin (Value: Longint);
begin
if Value <> fMin then
 begin
 fMin:=Value;
 if Value<0 then fMin:=0;
 if Position<Min then Position:=Min;
 Refresh;
 end;
end;


procedure ThhFader.SetMax (Value: Longint);
begin
if Value <> fMax then
 begin
 fMax:=Value;
 if Value<0 then fMax:=0;
 if Position>Max then Position:=Max;
 Refresh;
 end;
end;


procedure ThhFader.SetFrequency (Value: Longint);
begin
if Value <> fFrequency then
 begin
 fFrequency:=Value;
 if Value<1 then fFrequency:=1;
 Refresh;
 end;
end;


procedure ThhFader.SetTickLength (Value: Longint);
begin
if Value <> fTickLength then
 begin
 fTickLength:=Value;
 if Value<0 then fTickLength:=0;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobColor (Value: TColor);
begin
if Value <> fKnobColor then
 begin
 fKnobColor:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobBorderColor (Value: TColor);
begin
if Value <> fKnobBorderColor then
 begin
 fKnobBorderColor:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobWidth (Value: Longint);
begin
if Value <> fKnobWidth then
 begin
 fKnobWidth:=Value;
 if Value<0 then fKnobWidth:=0;
  if ( csdesigning in componentstate) then
  begin
  case Orientation of
   foHorizontal: if PaddingLeftRight<(Value div 2) then fPaddingLeftRight:=(Value div 2)+1;
   end;
  end;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobHeight (Value: Longint);
begin
if Value <> fKnobHeight then
 begin
 fKnobHeight:=Value;
 if Value<0 then fKnobHeight:=0;
 if ( csdesigning in componentstate) then
  begin
  case Orientation of
   foVertical: if PaddingTopBottom<(Value div 2) then fPaddingTopBottom:=(Value div 2)+1;
   end;
  end;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobBorderWidth (Value: Longint);
begin
if Value <> fKnobBorderWidth then
 begin
 fKnobBorderWidth:=Value;
 if Value<0 then fKnobBorderWidth:=0;
 Refresh;
 end;
end;


procedure ThhFader.SetTickSide (Value: TTickSide);
begin
if Value <> fTickSide then
 begin
 fTickSide:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetShowTickLine (Value: Boolean);
begin
if Value <> fShowTickLine then
 begin
 fShowTickLine:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetKnobGlyph(Value: TBitmap);
begin
 fKnobGlyph.Assign(Value);
 Invalidate;
 if fKnobGlyph.Empty=FALSE then
  begin
  fKnobWidth:=Value.Width;
  fKnobHeight:=Value.Height;
  fShowArrows:=FALSE;
  case Orientation of
   foVertical: if Value.Height/2 > PaddingTopBottom then fPaddingTopBottom:=Round(Value.Height/2)+1;
   foHorizontal: if Value.Width/2 > PaddingLeftRight then fPaddingLeftRight:=Round(Value.Width/2)+1;
   end;
  end;
 Refresh;
end;


procedure ThhFader.SetOrientation(Value: TOrientation);
var origwidth, origpadleft, origknobwidth: Longint;
begin
if Value <> fOrientation then
 begin
 fOrientation:=Value;
if ( csdesigning in componentstate) then
 begin
 origwidth:=Width;
 Width:=Height;
 Height:=origwidth;
 origpadleft:=PaddingLeftRight;
 fPaddingLeftRight:=PaddingTopBottom;
 fPaddingTopBottom:=origpadleft;
 origknobwidth:=KnobWidth;
 fKnobWidth:=KnobHeight;
 fKnobHeight:=origknobwidth;
 end;
 Refresh;
 end;
end;


procedure ThhFader.SetShowArrows (Value: Boolean);
begin
if Value <> fShowArrows then
 begin
 fShowArrows:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetBackGlyph (Value: TBitmap);
begin
 fBackGlyph.Assign(Value);
 Invalidate;
 if fBackGlyph.Empty=FALSE then
  begin
  Width:=Value.Width;
  Height:=Value.Height;
  end;
 Refresh;
end;


procedure ThhFader.SetScrollStroke (Value: Longint);
begin
if Value <> fScrollStroke then
 begin
 fScrollStroke:=Value;
 if Value<1 then fScrollStroke:=1;
 Refresh;
 end;
end;


procedure ThhFader.SetBevelInner (Value: TBevelForm);
begin
if Value <> fBevelInner then
 begin
 fBevelInner:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetBevelOuter (Value: TBevelForm);
begin
if Value <> fBevelOuter then
 begin
 fBevelOuter:=Value;
 Refresh;
 end;
end;


procedure ThhFader.SetBevelWidth (Value: Longint);
begin
if Value <> fBevelWidth then
 begin
 fBevelWidth:=Value;
 Refresh;
 end;
end;

//-----------------------------------------------------------------------------

procedure ThhFader.Paint;
var Bm: TBitmap;
    R, Rk: TRect;
    i: Integer;
    tdiv,tstart,KnobPos: Real;
    tposr,tposl: TPoint;
    TiSiLe,TiSiRe: Boolean;
    BevHiColor,BevLoColor: TColor;
begin
BevHiColor:=clBtnHighlight;
BevLoColor:=clBtnShadow;
TiSiLe:=FALSE;
TiSiRe:=FALSE;
case TickSide of
 tsLeftTop: TiSiLe:=TRUE;
 tsRightBottom: TiSiRe:=TRUE;
 tsBoth: begin TiSiLe:=TRUE; TiSiRe:=TRUE; end;
 end;
Bm:=TBitmap.Create;
Bm.Width:=Width;
bm.Height:=Height;
R.Left:=0;
R.Top:=0;
R.Right:=Width;
R.Bottom:=Height;

with Bm.Canvas do
 begin
 Brush.Color:=Color;
 FillRect(R);

if fBackGlyph.Empty = FALSE then StretchDraw(R,fBackGlyph);

case BevelOuter of
 bfRaised: Frame3D(Bm.Canvas,R,BevHiColor,BevLoColor,BevelWidth);
 bfLowered: Frame3D(Bm.Canvas,R,BevLoColor,BevHiColor,BevelWidth);
 end;
case BevelInner of
 bfRaised:  Frame3D(Bm.Canvas,R,BevHiColor,BevLoColor,BevelWidth);
 bfLowered:  Frame3D(Bm.Canvas,R,BevLoColor,BevHiColor,BevelWidth);
 end;

 case Orientation of
 foVertical: begin

  Pen.Color:=TickColor;
  MoveTo(Round(Width/2),PaddingTopBottom);
  if ShowTickLine then LineTo(Round(Width/2),Height-PaddingTopBottom);
  MoveTo(PaddingLeftRight,Height-PaddingTopBottom);
  if TiSiLe then LineTo(PaddingLeftRight+TickLength,Height-PaddingTopBottom);
  MoveTo(Width-PaddingLeftRight,Height-PaddingTopBottom);
  if TiSiRe then LineTo(Width-PaddingLeftRight-TickLength,Height-PaddingTopBottom);
  tdiv:=((Height-(2*PaddingTopBottom))/(Max-Min))*Frequency;
  tstart:=Height-PaddingTopBottom-tdiv;
  tposr.x:=Width-PaddingLeftRight;
  tposr.y:=Round(tstart);
  tposl.x:=PaddingLeftRight;
  tposl.y:=Round(tstart);
  for i:=(Min div Frequency) to ((Max-1) div Frequency) do
   begin
   MoveTo(tposl.x,tposl.y);
   if TiSiLe then LineTo(tposl.x+TickLength,tposl.y);
   MoveTo(tposr.x,tposr.y);
   if TiSiRe then LineTo(tposr.x-TickLength,tposr.y);
   tstart:=tstart-tdiv;
   tposr.y:=Round(tstart);
   tposl.y:=Round(tstart);
   end;

  KnobPos:=((Height-(2*PaddingTopBottom))*(Position-Min))/(Max-Min);
  Rk.Left:=Round((Width/2)-(KnobWidth/2));
  Rk.Right:=Rk.Left+KnobWidth;
  Rk.Bottom:=Height-PaddingTopBottom+Round(KnobHeight/2)-Round(KnobPos);
  Rk.Top:=Rk.Bottom-KnobHeight;
  Brush.Color:=KnobColor;
  Pen.Width:=KnobBorderWidth;

  if KnobBorderWidth = 0 then Pen.Color:=KnobColor else Pen.Color:=KnobBorderColor;

  if ShowArrows then
   case TickSide of
    tsLeftTop: Polygon([Point(Rk.Right,Rk.Top),Point(Rk.Left+Round(KnobHeight/2),Rk.Top),Point(Rk.Left,rk.Top+Round(KnobHeight/2)),Point(Rk.Left+Round(KnobHeight/2),Rk.Bottom),Point(Rk.Right,Rk.Bottom),Point(Rk.Right,Rk.Top)]);
    tsRightBottom: Polygon([Point(Rk.Left,Rk.Top),Point(Rk.Right-Round(KnobHeight/2),Rk.Top),Point(Rk.Right,rk.Top+Round(KnobHeight/2)),Point(Rk.Right-Round(KnobHeight/2),Rk.Bottom),Point(Rk.Left,Rk.Bottom),Point(Rk.Left,Rk.Top)]);
    tsBoth: Polygon([Point(Rk.Left,rk.Top+Round(KnobHeight/2)),Point(Rk.Left+Round(KnobHeight/2),Rk.Top),Point(Rk.Right-Round(KnobHeight/2),Rk.Top),Point(Rk.Right,rk.Top+Round(KnobHeight/2)),Point(Rk.Right-Round(KnobHeight/2),Rk.Bottom),Point(Rk.Left+Round(KnobHeight/2),Rk.Bottom),Point(Rk.Left,rk.Top+Round(KnobHeight/2))]);
    tsNone: Rectangle(Rk);
    end
  else Rectangle(Rk);

 end; //Ende von foVertical

 foHorizontal: begin

  Pen.Color:=TickColor;
  MoveTo(PaddingLeftRight,Round(Height/2));
  if ShowTickLine then LineTo(Width-PaddingLeftRight,Round(Height/2));
  MoveTo(PaddingLeftRight,PaddingTopBottom);
  if TiSiLe then LineTo(PaddingLeftRight,PaddingTopBottom+TickLength);
  MoveTo(PaddingLeftRight,Height-PaddingTopBottom);
  if TiSiRe then LineTo(PaddingLeftRight,Height-PaddingTopBottom-TickLength);
  tdiv:=((Width-(2*PaddingLeftRight))/(Max-Min))*Frequency;
  tstart:=PaddingLeftRight+tdiv;
  tposr.x:=Round(tstart);
  tposr.y:=Height-PaddingTopBottom;
  tposl.x:=Round(tstart);
  tposl.y:=PaddingTopBottom;
  for i:=(Min div Frequency) to ((Max-1) div Frequency) do
   begin
   MoveTo(tposl.x,tposl.y);
   if TiSiLe then LineTo(tposl.x,tposl.y+TickLength);
   MoveTo(tposr.x,tposr.y);
   if TiSiRe then LineTo(tposr.x,tposr.y-TickLength);
   tstart:=tstart+tdiv;
   tposr.x:=Round(tstart);
   tposl.x:=Round(tstart);
   end;

  KnobPos:=((Width-(2*PaddingLeftRight))*(Position-Min))/(Max-Min);
  Rk.Left:=PaddingLeftRight-Round(KnobWidth/2)+Round(KnobPos);
  Rk.Right:=Rk.Left+KnobWidth;
  Rk.Top:=Round((Height/2)-(KnobHeight/2));
  Rk.Bottom:=Rk.Top+KnobHeight;
  Brush.Color:=KnobColor;
  Pen.Width:=KnobBorderWidth;
  if KnobBorderWidth = 0 then Pen.Color:=KnobColor else Pen.Color:=KnobBorderColor;

  if ShowArrows then
   case TickSide of
    tsLeftTop: Polygon([Point(Rk.Right,Rk.Bottom),Point(Rk.Right,Rk.Top+Round(KnobWidth/2)),Point(Rk.Left+Round(KnobWidth/2),Rk.Top),Point(Rk.Left,Rk.Top+Round(KnobWidth/2)),Point(Rk.Left,Rk.Bottom),Point(Rk.Right,Rk.Bottom)]);
    tsRightBottom: Polygon([Point(Rk.Right,Rk.Top),Point(Rk.Right,Rk.Bottom-Round(KnobWidth/2)),Point(Rk.Left+Round(KnobWidth/2),Rk.Bottom),Point(Rk.Left,Rk.Bottom-Round(KnobWidth/2)),Point(Rk.Left,Rk.Top),Point(Rk.Right,Rk.Top)]);
    tsBoth: Polygon([Point(Rk.Left+Round(KnobWidth/2),Rk.Top),Point(Rk.Right,Rk.Top+Round(KnobWidth/2)),Point(Rk.Right,Rk.Bottom-Round(KnobWidth/2)),Point(Rk.Left+Round(KnobWidth/2),Rk.Bottom),Point(Rk.Left,Rk.Bottom-Round(KnobWidth/2)),Point(Rk.Left,Rk.Top+Round(KnobWidth/2)),Point(Rk.Left+Round(KnobWidth/2),Rk.Top)]);
    tsNone: Rectangle(Rk);
    end
  else Rectangle(Rk);

 end; //Ende von foHorizontal

 end; //Ende von case Orientation

 if fKnobGlyph.Empty = FALSE then StretchDraw(Rk,fKnobGlyph);

 end; //Ende von With Bm.Canvas

Canvas.CopyMode := cmSrcCopy;
Canvas.Draw(0, 0, Bm);
Bm.Destroy;
end;


procedure ThhFader.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
if Assigned(fOnMouseDown) then fOnMouseDown(Self,Button,Shift,X,Y);
SetFocus;
inherited MouseDown(Button, Shift, X, Y);
 case Orientation of
 foVertical: if (X>Round((Width/2)-(KnobWidth/2))) and (X<Round((Width/2)+(KnobWidth/2))) then MouseOn:=TRUE;
 foHorizontal: if (Y>Round((Height/2)-(KnobHeight/2))) and (Y<Round((Height/2)+(KnobHeight/2))) then MouseOn:=TRUE;
 end;
end;


procedure ThhFader.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
if Assigned(fOnMouseUp) then fOnMouseUp(Self,Button,Shift,X,Y);
inherited MouseUp(Button, Shift, X, Y);
MouseOn:=FALSE;
end;


procedure ThhFader.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
if Assigned(fOnMouseMove) then fOnMouseMove(Self,Shift,X,Y);
inherited MouseMove(Shift, X, Y);
if MouseOn = TRUE then
 begin
 UpdatePosition(X,Y)
 end;
end;


procedure ThhFader.UpdatePosition(X, Y: Integer);
var NegPix,PosPix,yps: Integer;
begin
case Orientation of
 foVertical: begin
             if Y>Height then yps:=Height else yps:=Y;
             NegPix:=Abs(yps-Height)-PaddingTopBottom;
             Position:=Round((NegPix*(Max-Min))/(Height-(2*PaddingTopBottom)))+Min;
             end;
 foHorizontal: begin
               PosPix:=X-PaddingLeftRight;
               Position:=Round((PosPix*(Max-Min))/(Width-(2*PaddingLeftRight)))+Min;
               end;
 end;              
if Position<Min then Position:=Min;
if Position>Max then Position:=Max;
Refresh;
end;


procedure ThhFader.FaderChange;
begin
if Assigned(fFaderChangeEvent) then fFaderChangeEvent(Self,Position);
end;


procedure ThhFader.WMMouseWheel(Var Message: TMessage);
begin
  if (csDesigning in ComponentState) then Exit;
  if ShortInt(Message.WParamHi) = -120 then Position:=Position-ScrollStroke
    else if ShortInt(Message.WParamHi) = 120 then Position:=Position+ScrollStroke;
end;


procedure Register;
begin
RegisterComponents('HansiSoft', [ThhFader]);
end;

end.
